package dev.adriankuta.flights.ui.home

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyListScope
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dev.adriankuta.flights.domain.types.AirportInfo
import dev.adriankuta.flights.domain.types.Country
import dev.adriankuta.flights.ui.home.components.AirportInfoItem
import dev.adriankuta.flights.ui.home.components.CountryItem

@Composable
internal fun StationsScreen(
    stationScreenViewModel: StationsScreenViewModel = hiltViewModel(),
) {
    val originAirports by stationScreenViewModel.originAirports.collectAsStateWithLifecycle()

    StationsScreen(
        originAirports = originAirports,
    )
}

@Composable
private fun StationsScreen(
    originAirports: OriginAirportsUiState,
) {
    LazyColumn(
        modifier = Modifier.fillMaxWidth(),
    ) {
        when (originAirports) {
            is OriginAirportsUiState.Error -> item { Text("Error") }
            is OriginAirportsUiState.Loading -> item { Text("Loading") }
            is OriginAirportsUiState.Success -> groupedAirports(originAirports.groupedAirports)
        }
    }
}

private fun LazyListScope.groupedAirports(
    data: Map<Country, List<AirportInfo>>,
) {
    data.keys.forEach { country ->
        stickyHeader { CountryItem(country) }
        items(data[country]!!) { airport ->
            AirportInfoItem(airport)
        }
    }
}
