package dev.adriankuta.flights.ui.home

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dev.adriankuta.flights.core.util.Result
import dev.adriankuta.flights.core.util.asResult
import dev.adriankuta.flights.domain.stations.ObserveAirportsGroupedByCountry
import dev.adriankuta.flights.domain.types.AirportInfo
import dev.adriankuta.flights.domain.types.Country
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
internal class StationsScreenViewModel @Inject constructor(
    observeAirportsGroupedByCountry: ObserveAirportsGroupedByCountry,
) : ViewModel() {

    val originAirports = originAirportsUiState(
        useCase = observeAirportsGroupedByCountry,
    ).stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(5_000),
        initialValue = OriginAirportsUiState.Loading,
    )
}

private fun originAirportsUiState(
    useCase: ObserveAirportsGroupedByCountry,
): Flow<OriginAirportsUiState> {
    return useCase()
        .asResult()
        .map { result ->
            when (result) {
                is Result.Error -> OriginAirportsUiState.Error(result.exception)
                is Result.Loading -> OriginAirportsUiState.Loading
                is Result.Success -> OriginAirportsUiState.Success(result.data.orEmpty())
            }
        }
}

internal sealed interface OriginAirportsUiState {
    data object Loading : OriginAirportsUiState
    data class Success(val groupedAirports: Map<Country, List<AirportInfo>>) : OriginAirportsUiState
    data class Error(val exception: Throwable) : OriginAirportsUiState
}

internal sealed interface DestinationAirportsUiState {
    data object Loading : DestinationAirportsUiState
    data class Success(val airports: List<AirportInfo>) : DestinationAirportsUiState
    data class Error(val exception: Throwable) : DestinationAirportsUiState
}
